@@ -3,7 +3,7 @@ package Catalyst::Engine::FastCGI;
 use strict;
 use base 'Catalyst::Engine::CGI';
 eval "use FCGI";
-die "Please install FCGI\n" if $@;
+die "Unable to load the FCGI module, you may need to install it:\n$@\n" if $@;
 
 =head1 NAME
 
@@ -22,30 +22,60 @@ This class overloads some methods from C<Catalyst::Engine::CGI>.
 Starts the FastCGI server.  If C<$listen> is set, then it specifies a
 location to listen for FastCGI requests;
 
-  Form            Meaning
-  /path           listen via Unix sockets on /path
-  :port           listen via TCP on port on all interfaces
-  hostname:port   listen via TCP on port bound to hostname
+=over 4
+
+=item /path
+
+listen via Unix sockets on /path
+
+=item :port
+
+listen via TCP on port on all interfaces
+
+=item hostname:port
+
+listen via TCP on port bound to hostname
+
+=back
 
 Options may also be specified;
 
-  Option          Meaning
-  leave_umask     Set to 1 to disable setting umask to 0
-                  for socket open
-  nointr          Do not allow the listener to be
-                  interrupted by Ctrl+C
-  nproc           Specify a number of processes for
-                  FCGI::ProcManager
-  pidfile         Specify a filename for the pid file
-  manager         Specify a FCGI::ProcManager sub-class
-  detach          Detach from console
+=over 4
+
+=item leave_umask
+
+Set to 1 to disable setting umask to 0 for socket open =item nointr
+
+Do not allow the listener to be interrupted by Ctrl+C
+
+=item nproc
+
+Specify a number of processes for FCGI::ProcManager
+
+=item pidfile
+
+Specify a filename for the pid file
+
+=item manager
+
+Specify a FCGI::ProcManager sub-class
+
+=item detach          
+
+Detach from console
+
+=item keep_stderr
+
+Send STDERR to STDOUT instead of the webserver
+
+=back
 
 =cut
 
 sub run {
     my ( $self, $class, $listen, $options ) = @_;
 
-    my $sock;
+    my $sock = 0;
     if ($listen) {
         my $old_umask = umask;
         unless ( $options->{leave_umask} ) {
@@ -65,9 +95,12 @@ sub run {
     $options ||= {};
 
     my %env;
-
+    my $error = \*STDERR; # send STDERR to the web server
+       $error = \*STDOUT  # send STDERR to stdout (a logfile)
+         if $options->{keep_stderr}; # (if asked to)
+    
     my $request =
-      FCGI::Request( \*STDIN, \*STDOUT, \*STDERR, \%env, $sock,
+      FCGI::Request( \*STDIN, \*STDOUT, $error, \%env, $sock,
         ( $options->{nointr} ? 0 : &FCGI::FAIL_ACCEPT_ON_INTR ),
       );
 
@@ -101,7 +134,16 @@ sub run {
 
     while ( $request->Accept >= 0 ) {
         $proc_manager && $proc_manager->pm_pre_dispatch();
+        
+        # If we're running under Lighttpd, swap PATH_INFO and SCRIPT_NAME
+        # http://lists.rawmode.org/pipermail/catalyst/2006-June/008361.html
+        # Thanks to Mark Blythe for this fix
+        if ( $env{SERVER_SOFTWARE} && $env{SERVER_SOFTWARE} =~ /lighttpd/ ) {
+            $env{PATH_INFO} ||= delete $env{SCRIPT_NAME};
+        }
+        
         $class->handle_request( env => \%env );
+        
         $proc_manager && $proc_manager->pm_post_dispatch();
     }
 }
@@ -117,6 +159,15 @@ sub write {
         $self->prepare_write($c);
         $self->{_prepared_write} = 1;
     }
+    
+    # XXX: We can't use Engine's write() method because syswrite
+    # appears to return bogus values instead of the number of bytes
+    # written: http://www.fastcgi.com/om_archive/mail-archive/0128.html
+    
+    # Prepend the headers if they have not yet been sent
+    if ( my $headers = delete $self->{_header_buf} ) {
+        $buffer = $headers . $buffer;
+    }
 
     # FastCGI does not stream data properly if using 'print $handle',
     # but a syswrite appears to work properly.
@@ -195,18 +246,22 @@ static, and dynamic.
 
 =head3 Standalone server mode
 
-    FastCgiExternalServer /tmp/myapp -socket /tmp/myapp.socket
-    Alias /myapp/ /tmp/myapp/
+    FastCgiExternalServer /tmp/myapp.fcgi -socket /tmp/myapp.socket
+    Alias /myapp/ /tmp/myapp/myapp.fcgi/
     
     # Or, run at the root
-    Alias / /tmp/myapp/
+    Alias / /tmp/myapp.fcgi/
     
     # Optionally, rewrite the path when accessed without a trailing slash
     RewriteRule ^/myapp$ myapp/ [R]
     
-The FastCgiExternalServer directive tells Apache that when serving /tmp/myapp
-to use the FastCGI application listenting on the socket /tmp/mapp.socket. 
-Note that /tmp/myapp does not need to exist -- it's a virtual file name.
+
+The FastCgiExternalServer directive tells Apache that when serving
+/tmp/myapp to use the FastCGI application listenting on the socket
+/tmp/mapp.socket.  Note that /tmp/myapp.fcgi does not need to exist --
+it's a virtual file name.  With some versions of C<mod_fastcgi> or
+C<mod_fcgid>, you can use any name you like, but most require that the
+virtual filename end in C<.fcgi>.
 
 It's likely that Apache is not configured to serve files in /tmp, so the 
 Alias directive maps the url path /myapp/ to the (virtual) file that runs the
@@ -269,6 +324,16 @@ application.
 For more information on using FastCGI under Apache, visit
 L<http://www.fastcgi.com/mod_fastcgi/docs/mod_fastcgi.html>
 
+=head3 Authorization header with mod_fastcgi or mod_cgi
+
+By default, mod_fastcgi/mod_cgi do not pass along the Authorization header,
+so modules like C<Catalyst::Plugin::Authentication::Credential::HTTP> will
+not work.  To enable pass-through of this header, add the following
+mod_rewrite directives:
+
+    RewriteCond %{HTTP:Authorization} ^(.+)
+    RewriteRule ^(.*)$ $1 [E=HTTP_AUTHORIZATION:%1,PT]
+
 =head2 Lighttpd
 
 These configurations were tested with Lighttpd 1.4.7.
@@ -309,8 +374,9 @@ values are disabled.  The above example would start 5 processes.
 =head3 Non-root configuration
     
 You can also run your application at any non-root location with either of the
-above modes.
+above modes.  Note the required mod_rewrite rule.
 
+    url.rewrite = ( "myapp\$" => "myapp/" )
     fastcgi.server = (
         "/myapp" => (
             "MyApp" => (
