@@ -7,6 +7,8 @@ use HTTP::Request;
 use Path::Class;
 use URI;
 use Class::Inspector;
+use Carp qw/croak/;
+use Cwd;
 
 =head1 NAME
 
@@ -22,28 +24,28 @@ See L<Catalyst>.
 
 =head2 appprefix($class)
 
-	MyApp::Foo becomes myapp_foo
+    MyApp::Foo becomes myapp_foo
 
 =cut
 
 sub appprefix {
     my $class = shift;
-    $class =~ s/\:\:/_/g;
+    $class =~ s/::/_/g;
     $class = lc($class);
     return $class;
 }
 
 =head2 class2appclass($class);
 
-    MyApp::C::Foo::Bar becomes MyApp
-    My::App::C::Foo::Bar becomes My::App
+    MyApp::Controller::Foo::Bar becomes MyApp
+    My::App::Controller::Foo::Bar becomes My::App
 
 =cut
 
 sub class2appclass {
     my $class = shift || '';
     my $appname = '';
-    if ( $class =~ /^(.*)::([MVC]|Model|View|Controller)?::.*$/ ) {
+    if ( $class =~ /^(.+?)::([MVC]|Model|View|Controller)::.+$/ ) {
         $appname = $1;
     }
     return $appname;
@@ -51,15 +53,15 @@ sub class2appclass {
 
 =head2 class2classprefix($class);
 
-    MyApp::C::Foo::Bar becomes MyApp::C
-    My::App::C::Foo::Bar becomes My::App::C
+    MyApp::Controller::Foo::Bar becomes MyApp::Controller
+    My::App::Controller::Foo::Bar becomes My::App::Controller
 
 =cut
 
 sub class2classprefix {
     my $class = shift || '';
     my $prefix;
-    if ( $class =~ /^(.*::[MVC]|Model|View|Controller)?::.*$/ ) {
+    if ( $class =~ /^(.+?::([MVC]|Model|View|Controller))::.+$/ ) {
         $prefix = $1;
     }
     return $prefix;
@@ -67,14 +69,14 @@ sub class2classprefix {
 
 =head2 class2classsuffix($class);
 
-    MyApp::C::Foo::Bar becomes C::Foo::Bar
+    MyApp::Controller::Foo::Bar becomes Controller::Foo::Bar
 
 =cut
 
 sub class2classsuffix {
     my $class = shift || '';
     my $prefix = class2appclass($class) || '';
-    $class =~ s/$prefix\:\://;
+    $class =~ s/$prefix\:://;
     return $class;
 }
 
@@ -89,7 +91,7 @@ Returns the environment name for class.
 
 sub class2env {
     my $class = shift || '';
-    $class =~ s/\:\:/_/g;
+    $class =~ s/::/_/g;
     return uc($class);
 }
 
@@ -97,7 +99,7 @@ sub class2env {
 
 Returns the uri prefix for a class. If case is false the prefix is converted to lowercase.
 
-    My::App::C::Foo::Bar becomes foo/bar
+    My::App::Controller::Foo::Bar becomes foo/bar
 
 =cut
 
@@ -105,9 +107,9 @@ sub class2prefix {
     my $class = shift || '';
     my $case  = shift || 0;
     my $prefix;
-    if ( $class =~ /^.*::([MVC]|Model|View|Controller)?::(.*)$/ ) {
+    if ( $class =~ /^.+?::([MVC]|Model|View|Controller)::(.+)$/ ) {
         $prefix = $case ? $2 : lc $2;
-        $prefix =~ s/\:\:/\//g;
+        $prefix =~ s{::}{/}g;
     }
     return $prefix;
 }
@@ -159,6 +161,7 @@ sub home {
 
             # find the @INC entry in which $file was found
             (my $path = $inc_entry) =~ s/$file$//;
+            $path ||= cwd() if !defined $path || !length $path;
             my $home = dir($path)->absolute->cleanup;
 
             # pop off /lib and /blib if they're there
@@ -170,8 +173,9 @@ sub home {
                 # clean up relative path:
                 # MyApp/script/.. -> MyApp
 
-                my ($lastdir) = $home->dir_list( -1, 1 );
-                if ( $lastdir eq '..' ) {
+                my $dir;
+                my @dir_list = $home->dir_list();
+                while (($dir = pop(@dir_list)) && $dir eq '..') {
                     $home = dir($home)->parent->parent;
                 }
 
@@ -199,7 +203,7 @@ sub home {
 
 Returns a prefixed action.
 
-    MyApp::C::Foo::Bar, yada becomes foo/bar/yada
+    MyApp::Controller::Foo::Bar, yada becomes foo/bar/yada
 
 =cut
 
@@ -220,10 +224,10 @@ sub request {
     my $request = shift;
     unless ( ref $request ) {
         if ( $request =~ m/^http/i ) {
-            $request = URI->new($request)->canonical;
+            $request = URI->new($request);
         }
         else {
-            $request = URI->new( 'http://localhost' . $request )->canonical;
+            $request = URI->new( 'http://localhost' . $request );
         }
     }
     unless ( ref $request eq 'HTTP::Request' ) {
@@ -232,22 +236,40 @@ sub request {
     return $request;
 }
 
-=head2 ensure_class_loaded($class_name)
+=head2 ensure_class_loaded($class_name, \%opts)
 
 Loads the class unless it already has been loaded.
 
+If $opts{ignore_loaded} is true always tries the require whether the package
+already exists or not. Only pass this if you're either (a) sure you know the
+file exists on disk or (b) have code to catch the file not found exception
+that will result if it doesn't.
+
 =cut
 
 sub ensure_class_loaded {
     my $class = shift;
+    my $opts  = shift;
 
-    return if Class::Inspector->loaded( $class ); # if a symbol entry exists we don't load again
+    croak "Malformed class Name $class"
+        if $class =~ m/(?:\b\:\b|\:{3,})/;
+
+    croak "Malformed class Name $class"
+        if $class =~ m/[^\w:]/;
+
+    croak "ensure_class_loaded should be given a classname, not a filename ($class)"
+        if $class =~ m/\.pm$/;
+
+    return if !$opts->{ ignore_loaded }
+        && Class::Inspector->loaded( $class ); # if a symbol entry exists we don't load again
 
     # this hack is so we don't overwrite $@ if the load did not generate an error
     my $error;
     {
         local $@;
-        eval "require $class";
+        my $file = $class . '.pm';
+        $file =~ s{::}{/}g;
+        eval { CORE::require($file) };
         $error = $@;
     }
 
@@ -286,6 +308,28 @@ sub merge_hashes {
     return \%merged;
 }
 
+=head2 env_value($class, $key)
+
+Checks for and returns an environment value. For instance, if $key is
+'home', then this method will check for and return the first value it finds,
+looking at $ENV{MYAPP_HOME} and $ENV{CATALYST_HOME}.
+
+=cut
+
+sub env_value {
+    my ( $class, $key ) = @_;
+
+    $key = uc($key);
+    my @prefixes = ( class2env($class), 'CATALYST' );
+
+    for my $prefix (@prefixes) {
+        if ( defined( my $value = $ENV{"${prefix}_${key}"} ) ) {
+            return $value;
+        }
+    }
+
+    return;
+}
 
 =head1 AUTHOR
 
